<?php

namespace App\Services\Repositories;

use App\Http\Controllers\Base;
use App\Http\Resources\ContactUsFilterResource;
use App\Mail\ContactUsReplay;
use App\Models\Category;
use App\Models\ContactUs;
use App\Services\Interface\ContactUsInterface;

use App\Models\User;
use App\Support\Collection;
use Exception;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Mail;
use Illuminate\Support\Facades\Validator;

class ContactUsRepository implements ContactUsInterface
{

    public function add($data)
    {
        try {
            $validator =  Validator::make($data->all(), [
                'name' => 'required',
                'email' => 'required',
                'subject' => 'required',
                'message' => 'required',
            ]);

            if ($validator->fails()) return Base::validation($validator);

            $contact_us = new ContactUs();

            if (isset($data->user_id)) {
                $user = User::where('id', $data->user_id)->first();
                if (!$user) return Base::error('User not found');
                else $contact_us->user_id = $user->id;
            }

            $contact_us->name = $data['name'];
            $contact_us->email = $data['email'];
            $contact_us->subject = $data['subject'];
            $contact_us->message = $data['message'];
            $contact_us->save();

            return Base::success('Contact us form submitted');
        } catch (Exception $e) {
            return Base::exception_fail($e);
        }
    }


    public function adminContactUsIndex($request)
    {
        try {
            $take = isset($request['take']) ? $request['take'] : 20;
            $contact_us = ContactUs::orderBy('id', 'desc')->with('user')->paginate($take);

            // $search = isset($request['search']) ? $request['search'] : null;
            // $contact_us = ContactUs::where('name', 'like', '%' .$search. '%')->get();

            return Base::pass("All Contact Us list", $contact_us);
        } catch (Exception $e) {
            return Base::exception_fail($e);
        }
    }

    public function adminContactUsSearch($request)
    {
        try {
            $search = isset($request['search']) ? $request['search'] : null;
            $contact_us = ContactUs::where('name', 'like', '%' . $search . '%')->get();

            return Base::pass("All Contact Us list", $contact_us);
        } catch (Exception $e) {
            return Base::exception_fail($e);
        }
    }

    public function adminContactUsShow($data)
    {
        try {
            $contact_us = ContactUs::where('id', $data['id'])
                ->with(['user'])
                ->first();

            $contact_us->isSeen = true;
            $contact_us->isNew = false;
            $contact_us->save();

            return Base::pass("contact us replay details", $contact_us);
        } catch (Exception $e) {
            return Base::exception_fail($e);
        }
    }

    public function adminContactUsDelete($data)
    {
        try {
            $contact_us = ContactUs::where('id', $data['id'])->first();
            $contact_us->delete();

            return Base::pass(null, $contact_us);
        } catch (Exception $e) {
            return Base::exception_fail($e);
        }
    }

    public function updateUser($user, $data)
    {
        try {
            if (isset($data['replay'])) $user->replay = $data['replay'];
            $user->save();

            // $details = [
            //     'replay' => $data['replay']
            // ];

            // Mail::to('info@unimassbd.com')->send(new ContactUsReplay($details));

            return Base::pass("User profile updated", $user);
        } catch (Exception $e) {
            return Base::exception_fail($e);
        }
    }

    public function adminContactUsUpdate($request)
    {
        try {
            $user = ContactUs::find($request['id']);
            if (!$user) return Base::error("User not found");
            $data = $this->updateUser($user, $request);

            return $data->success ? Base::pass($data->message, $data->data) : Base::fail($data->message);
        } catch (Exception $e) {
            return Base::exception($e);
        }
    }

    public function reply($request)
    {
        try {
            $validator =  Validator::make($request->all(), [
                'id' => 'required',
                'reply' => 'required',
            ]);

            if ($validator->fails()) return Base::validation($validator);
            $contact_us = ContactUs::find($request->id);

            if (!$contact_us) return Base::error('Contact us not found');

            if ($contact_us->isReplied) return Base::error('Already replied');

            $contact_us->replay = $request->reply;
            $contact_us->isReplied = true;
            $contact_us->isNew = false;
            $contact_us->isSeen = true;

            $details = [
                'title' => 'Reply from Gifty',
                'body' => $request->reply
            ];

            Mail::to($contact_us->email)->send(new ContactUsReplay($details));

            $contact_us->save();

            return Base::success('Reply sent successfully', $contact_us);
        } catch (Exception $e) {
            return Base::exception($e);
        }
    }

    public function filter($request)
    {
        try {
            $take = isset($request['take']) ? $request['take'] : 20;
            $search = isset($request['search_key']) ? $request['search_key'] : '';

            $contact_us = ContactUsFilterResource::collection(ContactUs::orderBy('id', 'desc')
                ->when(isset($search), function ($q) use ($search) {
                    return $q->where('name', 'like', '%' . $search . '%')
                        ->orWhere('email', 'like', '%' . $search . '%');
                })->with('user')->get());

            $contact_us = $contact_us->when(isset($request->role), function ($q) use ($request) {
                if ($request->role === 'customer') return $q->where('user.role', 'customer');
                if ($request->role === 'shop') return $q->where('user.role', 'shop');
                if ($request->role === 'guest') return $q->where('user.role', null);
            })->when(isset($request['is_read']), function ($q) use ($request) {
                if ($request['is_read'] === 'true') return $q->where('isSeen', true);
                if ($request['is_read'] === 'false') return $q->where('isSeen', false);
            })->when(isset($request['is_replied']), function ($q) use ($request) {
                if ($request['is_replied'] === 'true') return $q->where('isReplied', true);
                if ($request['is_replied'] === 'false') return $q->where('isReplied', false);
            })->when(isset($request->start_date) && isset($request->end_date), function ($q) use ($request) {
                return $q->whereBetween('created_at', [$request->start_date . ' 00:00:00', $request->end_date . ' 23:59:59']);
            });

            $contact_us = (new Collection($contact_us))->values()->paginate($take);

            return Base::pass("Contact Us filter result!", $contact_us);
        } catch (Exception $e) {
            return Base::exception_fail($e);
        }
    }
}
