<?php

namespace App\Services\Repositories;

use App\Http\Controllers\Base;
use App\Http\Resources\ShopResource;
use App\Models\PostCard;
use App\Models\Shop;
use App\Models\ShopBoost;
use App\Services\Interface\ShopInterface;
use App\Models\User;
use App\Support\Collection;
use App\Traits\GlobalTrait;
use Exception;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Validator;

class ShopRepository implements ShopInterface
{
    use GlobalTrait;

    public function filter()
    {
        try {
            $shops = Shop::with('user')->whereHas('user')->get();
            $shops = ShopResource::collection(Shop::orderBy('rate', 'desc')->with('User')->get());

            return Base::pass(null, $shops);
        } catch (Exception $e) {
            return Base::exception_fail($e);
        }
    }

    public function appHome($data)
    {
        try {
            // todo: apply scope
            $shops = ShopResource::collection(Shop::orderBy('rate', 'desc')->with('User')->take($data)->get());

            $shops = collect($shops);

            $shops = $shops->where('is_active', 1)->values()->all();

            return Base::pass(null, $shops);
        } catch (Exception $e) {
        }
    }

    public function appHome2($data)
    {
        try {
            $shops = Shop::join('users', 'users.id', '=', 'shops.user_id')
                ->select('shops.*', 'users.name', 'users.email', 'users.phone', 'users.address', 'users.image', 'users.image_url', 'users.role', 'users.lang_code')
                // ->withCount('postCards2 as shop_cards_count')
                ->withCount('postCardCounter as shop_cards_count')
                ->orderBy('rate', 'desc')
                ->where('users.is_active', 1)
                ->take($data)
                ->get();
            return Base::pass(null, $shops);
        } catch (Exception $e) {
            return Base::exception_fail($e->getMessage());
        }
    }

    public function getAllShopsFoo($request)
    {
        try {
            $take = isset($request['take']) ? $request['take'] : 20;
            $is_boosted = isset($request['is_boosted']) ? $request['is_boosted'] : false;
            $boosted_shop = [];
            if ($is_boosted == true) {
                $boosted_shop = ShopBoost::with('shop')->get()->pluck('shop_id')->toArray();
            }
            $shops = Shop::whereNotIn('user_id', $boosted_shop)->with('userData')->withCount('postCards2 as shop_cards_count', 'orders', 'pendingOrders')->paginate($take);
            $shops->data = $shops->makeHidden(['deleted_at', 'created_at', 'updated_at']);

            return Base::pass('All shops information', $shops); // DONE
        } catch (Exception $e) {
            // return $e;
            return Base::exception_fail($e->getMessage());
        }
    }


    public function getShopFilter($request)
    {
        try {
            $is_mobile = $request->is_mobile ?? 0;
            $take = isset($request['take']) ? $request['take'] : 20;
            
            if ($is_mobile) {
                $search = isset($request['search']) ? $request['search'] : null;

                $shops = Shop::join('users', 'users.id', '=', 'shops.user_id')
                    ->with('userData')
                    ->whereHas('userData', function ($q) use ($search) {
                        $q->where('name', 'like', '%' . $search . '%')
                        ->orWhere('email', 'like', '%' . $search . '%');
                    })
                    ->select('users.name', 'users.email', 'users.phone', 'users.image', 'users.image_url', 'users.address', 'users.role', 'users.lang_code', 'users.is_active', 'shops.details', 'shops.rate', 'shops.user_id',)
                    ->withCount('postCardCounter as shop_cards_count')
                    ->where('users.is_active', 1)
                    ->paginate($take);

                // $shops = ShopResource::collection(Shop::orderBy('rate', 'desc')->with('User')->paginate($take));


                // $shops = (new Collection($shops))->paginate($take);
            } else {
                $search = isset($request['search']) ? $request['search'] : null;
                $shops = Shop::join('users', 'users.id', '=', 'shops.user_id')
                    ->with('userData')
                    ->whereHas('userData', function ($q) use ($search) {
                        $q->where('name', 'like', '%' . $search . '%')
                        ->orWhere('email', 'like', '%' . $search . '%');
                    })
                    ->withCount('postCardCounter as shop_cards_count', 'orders', 'pendingOrders', 'completedOrders')
                    ->when(isset($request['is_active']), function ($q) use($request){
                        $q->where('users.is_active', [$request['is_active']]);
                    })
                    // ->where('users.is_active', 1)
                    ->paginate($take);
            }

            return Base::pass('All shops information', $shops); // DONE
        } catch (Exception $e) {
            return Base::exception_fail($e->getMessage());
        }
    }

    public function addNewShopFoo(Request $request)
    {
        try {
            $validator = Validator::make($request->all(), [
                'name' => 'required|string|max:255',
                'address' => 'required|string|max:255',
                'phone' => 'required|string|max:255',
                'email' => 'required|string|email|max:255|unique:users',
                'password' => 'required|string|min:6|confirmed',
            ]);

            if ($validator->fails()) return Base::validation($validator);

            $user = new User();
            $user->name = $request->name;
            $user->email = $request->email;
            $user->address = $request->address;
            $user->phone = $request->phone;
            $user->password = bcrypt($request->password);
            $user->role = 'shop';
            $user->is_verified = 1;
            if (isset($request['image_url'])) $user->image_url = GlobalTrait::imageUpload($request['image_url'], 'shops');
            $user->save();

            $shop = new Shop();
            $shop->user_id = $user->id;
            $shop->details = $request->details ?? null;
            $shop->save();



            $shop_index = $this->getAllShopsFoo($request);

            $data =  $shop_index->data;

            return Base::pass('Shop added successfully', $data); // DONE
        } catch (Exception $e) {
            // return $e;
            return Base::exception_fail($e);
        }
    }

    public function updateShopFoo(Request $request)
    {
        try {
            $validator = Validator::make($request->all(), [
                'user_id' => 'required|exists:users,id',
                'name' => 'string|max:255',
                'address' => 'string|max:255',
                'phone' => 'string|max:255',
                'email' => 'string|email|max:255|unique:users',
                'password' => 'string|min:6|confirmed',
            ]);

            if ($validator->fails()) return Base::validation($validator);

            $user = User::find($request->user_id);
            if (!$user) return Base::fail("User not found",);
            $shop = Shop::where('user_id', $user->id)->first();

            if (!$shop) return Base::error('Shop not found'); // DONE

            if (isset($request['name'])) $user->name = $request->name;
            if (isset($request['address'])) $user->address = $request->address;
            if (isset($request['phone'])) $user->phone = $request->phone;
            if (isset($request['email'])) $user->email = $request->email;
            if (isset($request['password'])) $user->password = bcrypt($request->password);
            if (isset($request['image'])) {
                if ($user->image_url) {
                    $image_path = public_path($user->image_url);
                    if (file_exists($image_path)) unlink($image_path);
                }
                $user->image_url = GlobalTrait::imageUpload($request['image'], 'shops');
            }

            if (isset($request['details'])) $shop->details = $request->details;
            if (isset($request['is_active'])) $user->is_active = $request['is_active'] == true ? 1 : 0;

            $user->save();
            $shop->save();

            $shop_index = $this->showShopFoo($request);

            $data =  $shop_index->data;

            return Base::pass('Shop updated successfully', $data); // DONE
        } catch (Exception $e) {
            // return $e;
            return Base::exception_fail($e);
        }
    }

    public function deleteShopFoo(Request $request)
    {
        try {
            $validator = Validator::make($request->all(), [
                'user_id' => 'required|exists:users,id',
            ]);

            if ($validator->fails()) return Base::validation($validator);

            $user = User::where([['id', $request->user_id], ['role', 'shop']])->first();

            if (!$user) return Base::fail('Shop not found'); // DONE

            $pending_orders = $user->shop->pendingOrders->count();

            if ($pending_orders > 0) return Base::fail('Shop has pending orders, unable to delete shop'); // DONE
            $shop = $user->shop;
            $user->delete();
            $shop->delete();


            $shop_index = $this->getAllShopsFoo($request);
            $data =  $shop_index->data;

            return Base::pass('Shop deleted successfully', $data); // DONE
        } catch (Exception $e) {
            // return $e;
            return Base::exception_fail($e);
        }
    }

    public function showShopFoo(Request $request)
    {
        try {
            //Code will be here
            $validator = Validator::make($request->all(), [
                'user_id' => 'required|exists:users,id',
            ]);

            if ($validator->fails()) return Base::validation($validator);

            $user = User::where([['id', $request->user_id], ['role', 'shop']])->with('shop')->first();

            if (!$user) return Base::fail('Shop not found'); // DONE

            $total_cards = $user->postCards->count();
            $total_orders = $user->shopPostCardsWithOrders->sum('orders_count');
            $total_ongoing_orders = $user->shopPostCardsWithPendingOrders->sum('pending_orders_count');
            $total_complete_orders = $user->shopPostCardsWithSoldOrders->sum('sold_count');

            $data = [
                "shop" => $user->unsetRelation('postCards')
                    ->unsetRelation('shopPostCardsWithOrders')
                    ->unsetRelation('shopPostCardsWithPendingOrders')
                    ->unsetRelation('shopPostCardsWithSoldOrders'),
                "total_cards" => $total_cards,
                "total_orders" => $total_orders,
                "total_ongoing_orders" => $total_ongoing_orders,
                "total_complete_orders" => $total_complete_orders,
            ];

            return Base::pass("Shop details", $data); // DONE
        } catch (Exception $e) {
            return Base::exception_fail($e);
        }
    }
}
