<?php

namespace App\Services\Repositories;

use App\Http\Controllers\Base;
use App\Http\Controllers\NotificationController;
use App\Http\Requests\TogglePushOrEmailNotificationRequest;
use App\Http\Resources\UserResource;
use App\Jobs\SendOtpJob;
use App\Models\GeneralInfo;
use App\Models\RewardPoint;
use App\Models\Setting;
use App\Models\Shop;
use App\Services\Interface\UserInterface;
use App\Models\User;
use App\Models\UserInfo;
use App\Traits\GlobalTrait;
use Exception;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Hash;
use Illuminate\Support\Facades\Validator;
use Laravel\Socialite\Facades\Socialite;

class UserRepository implements UserInterface
{

    use GlobalTrait;

    public function login($credentials, $role)
    {
        try {
            // ! Checking Credentials
            if (!Auth::attempt($credentials)) return Base::fail('Invalid Credentials'); // DONE
            // ! Checking app role
            if (Auth::user()->role != $role) return Base::fail("You can not login this app with this account"); // DONE

            $accessToken = Auth::user()->createToken('authToken')->accessToken;

            $data = [
                'token' => $accessToken,
                'user' => new UserResource(Auth::user()),
            ];

            // ! Checking if verified
            if (Auth::user()->is_verified) {
                if(Auth::user()->is_active){
                    return Base::pass('Login Successful', $data);
                }
                else {
                    return Base::fail('Account is deactivate');
                }
            } else {
                $is_otp_sent = $this->sendOTP(Auth::user());
                if (!$is_otp_sent) return Base::error('Login Failed, Please try again'); // DONE

                return Base::pass('Logged in successful, Please verify your account', $data); // DONE
            }
        } catch (Exception $e) {
            return Base::exception_fail($e);
        }
    }

    public function socialLogin($request)
    {
        try {
            
            $social_user = Socialite::driver($request['provider'])->userFromToken($request['token']);
            $data = response()->json($social_user);

            $user_name = $data->getData()->name ?? GlobalTrait::formateUserName($data->getData()->email);
            $user_email =  $data->getData()->email;
            $user_id =  $data->getData()->id;
            $user_image =  $data->getData()->avatar;

            // $randomPassword = rand(10000000000, 99999999999);
            $str_result = '0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz';
            $randomPassword = substr(str_shuffle($str_result), 0, 10);
            

            $role = $request['role'];

            $user = User::where(["login_id" => $user_id, "login_type" => $request['provider']])->first();

            if ($user != null) {
                if ($user->is_active == 0) return Base::fail('Account is deactivate');
                $accessToken = $user->createToken('authToken')->accessToken;
                $user_data = new UserResource($user);
            } else {
                if ($user_email != null) {

                    $user_find = User::where("email", $user_email)->first();
                    if ($user_find){

                        // return Base::fail("User already exist with this email.");
                        $accessToken = $user_find->createToken('authToken')->accessToken;
                        $user_data = new UserResource($user_find);
                    } 
                    else {

                        if (isset($request->refer_code)) {
                            $settings = Setting::all();
                            $is_referral = $settings->where('type', 'is_referral')->first()->value;
                            if ($is_referral == 0) return Base::error("Referral is disabled"); // DONE
                            $refer_user = User::where('refer_code', $request->refer_code)->first();
                            if (!$refer_user) return Base::error("Wrong refer code"); // DONE
                        }
                        $new_user = new User();

                        $new_user->name = $user_name;
                        $new_user->email = $user_email ?? null;
                        $new_user->password = Hash::make($randomPassword);
                        $new_user->login_id = $user_id;
                        $new_user->image_url = $user_image;
                        $new_user->login_type = $request['provider'];
                        $new_user->is_active = true;
                        $new_user->is_verified = true;
                        $new_user->role = $role;
                        $new_user->refer_code = $this->generateReferCode();
                        $new_user->refer_by = $request['refer_code'] ?? null;
                        $new_user->save();

                        $accessToken = $new_user->createToken('authToken')->accessToken;
                        $new_user->refresh();

                        if ($new_user->refer_by != null) {
                            $settings = Setting::all();
                            $is_referral = $settings->where('type', 'is_referral')->first()->value;
                            if ($is_referral == 1) {
                                $referral_value = $settings->where('type', 'referral_point')->first()->value;
                                $refer_user = User::where('refer_code', $new_user->refer_by)->first();
                                $refer_user->point = $refer_user->point + $referral_value;


                                $reward_point = new RewardPoint();
                                $reward_point->user_id = $refer_user->id;
                                $reward_point->point = $referral_value;
                                $reward_point->type = 'referral';
                                $reward_point->description = 'Referral point';
                                $reward_point->save();

                                $refer_user->save();

                                $refer_notification = [
                                    'from_id' => $new_user->id,
                                    'to_id' => $refer_user->id,
                                    'type' => 'referral',
                                    'title_key' => 'referral_title',
                                    'description_key' => 'referral_description',
                                    'value' => $referral_value,
                                ];
                                NotificationController::setNotification($refer_notification);
                            }
                        }

                        $user_data = new UserResource($new_user);
                    }
                } else {

                    if (isset($request->refer_code)) {
                        $settings = Setting::all();
                        $is_referral = $settings->where('type', 'is_referral')->first()->value;
                        if ($is_referral == 0) return Base::error("Referral is disabled"); // DONE
                        $refer_user = User::where('refer_code', $request->refer_code)->first();
                        if (!$refer_user) return Base::error("Wrong refer code"); // DONE
                    }
                    $new_user = new User();

                    $new_user->name = $user_name;
                    $new_user->email = $user_email ?? null;
                    $new_user->password = Hash::make($randomPassword);
                    $new_user->login_id = $user_id;
                    $new_user->image_url = $user_image;
                    $new_user->login_type = $request['provider'];
                    $new_user->is_active = true;
                    $new_user->is_verified = true;
                    $new_user->role = $role;
                    $new_user->refer_code = $this->generateReferCode();
                    $new_user->refer_by = $request['refer_code'] ?? null;
                    $new_user->save();

                    $accessToken = $new_user->createToken('authToken')->accessToken;
                    $new_user->refresh();

                    if ($new_user->refer_by != null) {
                        $settings = Setting::all();
                        $is_referral = $settings->where('type', 'is_referral')->first()->value;
                        if ($is_referral == 1) {
                            $referral_value = $settings->where('type', 'referral_point')->first()->value;
                            $refer_user = User::where('refer_code', $new_user->refer_by)->first();
                            $refer_user->point = $refer_user->point + $referral_value;


                            $reward_point = new RewardPoint();
                            $reward_point->user_id = $refer_user->id;
                            $reward_point->point = $referral_value;
                            $reward_point->type = 'referral';
                            $reward_point->description = 'Referral point';
                            $reward_point->save();

                            $refer_user->save();

                            $refer_notification = [
                                'from_id' => $new_user->id,
                                'to_id' => $refer_user->id,
                                'type' => 'referral',
                                'title_key' => 'referral_title',
                                'description_key' => 'referral_description',
                                'value' => $referral_value,
                            ];
                            NotificationController::setNotification($refer_notification);
                        }
                    }

                    $user_data = new UserResource($new_user);
                }
            }
            $data1 = [
                'token' => $accessToken,
                'user' => $user_data,
            ];
            return Base::pass('Login Successful', $data1);
        } catch (Exception $e) {
            return Base::exception_fail($e->getMessage() . " " . $e->getFile() . " " . $e->getLine());
        }
    }

    public function generateReferCode()
    {
        $str_result = '0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz';
        $random = substr(str_shuffle($str_result), 0, 6);

        $found_duplicate = User::where('refer_code', $random)->first();
        if ($found_duplicate) $this->generateReferCode();

        return $random;
    }


    public function createUser($data)
    {
        try {
            $user = new User();
            $user->name = $data['name'];
            $user->email = strtolower($data['email']);
            $user->password = Hash::make($data['password']);
            $user->role = $data['role'];
            $user->refer_code = $this->generateReferCode();
            $user->refer_by = $data['refer_code'] ?? null;
            $user->save();

            return Base::pass(null, $user);
        } catch (Exception $e) {
            return Base::exception_fail($e);
        }
    }



    public function createShop($user_id)
    {
        try {
            $shop = new Shop();
            $shop->user_id = $user_id;
            $shop->save();

            return Base::pass(null, $shop);
        } catch (Exception $e) {
            return Base::exception_fail($e);
        }
    }


    public function sendOtp($user)
    {
        try {
            $generateOTP = rand(1000, 9999);
            $user->otp = Hash::make($generateOTP);
            $user->otp_created_at = Base::now();
            $user->save();

            $details = [
                'greeting' => 'Hi ' . $user->name,
                'body' => 'Here is your verification code that\'s valid for only 5 Mins. If its not you please ignore it.',
                'otp' => 'Verification Code: ' . $generateOTP,
            ];
            // $user->notify(new VerifyOTP($details));
            dispatch(new SendOtpJob($user, $details));

            return Base::pass(null, $user);
        } catch (Exception $e) {
            return Base::exception_fail($e);
        }
    }


    public function getUserProfile()
    {
        try {
            $data = new UserResource(Auth::user());

            return Base::pass(null, $data);
        } catch (Exception $e) {
            return Base::exception_fail($e);
        }
    }


    public function updateUser($user, $data)
    {
        try {
            if (isset($data['image_url'])) {
                $img = GlobalTrait::imageUpload($data['image_url'], 'users');
                $user->image_url = $img;
            }

            if (isset($data['name'])) $user->name = $data['name'];
            if (isset($data['email'])) $user->email = $data['email'];
            if (isset($data['phone'])) $user->phone = $data['phone'];
            if (isset($data['address'])) $user->address = $data['address'];
            if (isset($data['post_code'])) $user->post_code = $data['post_code'];
            if (isset($data['city'])) $user->city = $data['city'];

            $user->save();

            return Base::pass("User profile updated", $user); // DONE
        } catch (Exception $e) {
            return Base::exception_fail($e);
        }
    }


    public function updatePassword($user, $password)
    {
        try {
            $user->password = Hash::make($password);
            $user->save();
            $data = new UserResource($user);

            return Base::pass(null, $data);
        } catch (Exception $e) {
            return Base::exception_fail($e);
        }
    }

    public function adminCustomerChangePassword(Request $request)
    {
        try {
            $validator = Validator::make($request->all(), [
                'password' => 'required|confirmed|min:6',
            ]);
            if ($validator->fails()) return Base::validation($validator);

            $user = User::find($request['user_id']);

            if (!$user) return Base::fail("User not found");

            $user->password = Hash::make($request['password']);
            $user->save();
            $data = new UserResource($user);

            return Base::pass(null, $data);
        } catch (Exception $e) {
            return Base::exception_fail($e);
        }
    }

    public function updateLanguage($user, $code)
    {
        try {
            $country_code = null;
            switch ($code){
                case 'en':
                    $country_code = 'US';
                    break;
                case 'bn':
                    $country_code = 'BD';
                    break;
                case 'ja':
                    $country_code = 'JP';
                    break;
                case 'zh':
                    $country_code = 'CN';
                    break;
            }
            $user->lang_code = $code;
            $user->country_code = $country_code;
            $user->save();
            $data = new UserResource($user);

            return Base::pass(null, $data);  
        } catch (Exception $e) {
            return Base::exception_fail($e);
        }
    }

    public function togglePushNotification($user)
    {
        try {
            $user->is_push_notification = !$user->is_push_notification;
            $data = new UserResource($user);

            return Base::pass(null, $data);
        } catch (Exception $e) {
            return Base::exception_fail($e);
        }
    }

    public function toggleEmailNotification($user)
    {
        try {
            $user->is_email_notification = !$user->is_email_notification;
            $data = new UserResource($user);

            return Base::pass(null, $data);
        } catch (Exception $e) {
            return Base::exception_fail($e);
        }
    }

    public function togglePushOrEmailNotification(TogglePushOrEmailNotificationRequest $request)
    {
        try {
            $user = Auth::user();

            if ($request->type == 'push_notification') $user->is_push_notification = !$user->is_push_notification;
            else  $user->is_email_notification = !$user->is_email_notification;

            $user->save();

            $data = [
                'is_push_notification' => $user->is_push_notification == 1 ? 1 : 0,
                'is_email_notification' => $user->is_email_notification == 1 ? 1 : 0,
            ];

            return Base::success('Settings Changed', $data); // DONE
        } catch (Exception $e) {
            return Base::exception($e);
        }
    }

    public function delete($user, $request)
    {
        try {
            // $user->delete();
            $user->name = "Gifty User";
            $user->email = "Gifty User" . $user->id;
            $user->password = "Gifty User";
            $user->login_id = null;
            $user->image = null;
            $user->image_url = null;
            $user->phone = null;
            $user->address = null;
            $user->is_verified = false;
            $user->is_active = false;
            $user->is_push_notification = false;
            $user->is_email_notification = false;
            $user->refer_by = null;
            $user->refer_code = null;
            $user->city = null;
            $user->post_code = null;
            $user->delete_reason = $request->reason;
            $user->save();

            return Base::pass(null, $user);
        } catch (Exception $e) {
            return Base::exception_fail($e);
        }
    }

    public function deleteText()
    {
        try {
            //Code will be here
            // if (Auth::user()->role == 'customer') {
            //     $data = "<h2>Delete profile message for customer</h2>";
            // } else {
            //     $data = "<h2>Delete profile message for shop</h2>";
            // }


            // if (Auth::user()->role == 'customer') {
            //     if (Auth::user()->lang_code == 'en') {
            //         $delete_message = GeneralInfo::where(['type' => 'delete-message', 'lang' => 'en'])->first();
            //         $data = $delete_message->info;
            //     } else {
            //         $delete_message = GeneralInfo::where(['type' => 'delete-message', 'lang' => 'ja'])->first();
            //         $data = $delete_message->info;
            //     }
            // } else {
            //     if (Auth::user()->lang_code == 'en') {
            //         $delete_message = GeneralInfo::where(['type' => 'delete-message', 'lang' => 'en'])->first();
            //         $data = $delete_message->info;
            //     } else {
            //         $delete_message = GeneralInfo::where(['type' => 'delete-message', 'lang' => 'ja'])->first();
            //         $data = $delete_message->info;
            //     }
            // }

            $delete_message = GeneralInfo::where(['type' => 'delete-message', 'lang' => Auth::user()->lang_code])->first();
            $data = $delete_message->info;

            return Base::pass(null, $data);
        } catch (Exception $e) {
            return Base::exception_fail($e);
        }
    }


    //* Admin

    public function adminCustomerIndex($request)
    {
        try {
            //Code will be here
            $take = isset($request['take']) ? $request['take'] : 20;
            $customer = User::customer()
                ->withCount('customerOrders as total_orders')
                ->with('customerLatestOrders', function ($q) {
                    // return $q->pivot(['card_name']);
                })
                ->paginate($take);
            // ->makeVisible([
            //     'is_verified',
            //     'is_active',
            // ]);
            return Base::pass("All Customer list", $customer); // DONE
        } catch (Exception $e) {
            return Base::exception_fail($e);
        }
    }

    public function adminCustomerFilter($request)
    {
        try {
            //Code will be here
            $take = isset($request['take']) ? $request['take'] : 20;
            $search = isset($request['search']) ? $request['search'] : null;
            $customer = User::customer()
                ->where('name', 'like', '%' . $search . '%')
                ->withCount('customerOrders as total_orders')
                ->with('customerLatestOrders', function ($q) {
                    // return $q->pivot(['card_name']);
                })
                ->paginate($take);
            // ->makeVisible([
            //     'is_verified',
            //     'is_active',
            // ]);
            return Base::pass("All Customer list", $customer); // DONE
        } catch (Exception $e) {
            return Base::exception_fail($e);
        }
    }

    public function adminCustomerShow($data)
    {
        try {
            //Code will be here
            $customer = User::customer()
                ->where('id', $data['id'])
                ->with(['userInfo'])
                ->first();
            return Base::pass("Customer details", $customer); // DONE
        } catch (Exception $e) {
            return Base::exception_fail($e);
        }
    }

    public function adminStatusToggle($data)
    {
        try {
            //Code will be here
            $user = User::find($data['user_id']);
            $user->is_active = !$user->is_active;
            $user->save();
            $user->refresh();
            $data = new UserResource($user);

            return Base::pass("User status changed", $data); // DONE
        } catch (Exception $e) {
            return Base::exception($e);
        }
    }

    public function adminVidStatusToggle($data)
    {
        try {
            //Code will be here
            $user = User::find($data['user_id']);
            // $user->is_active = !$user->is_active;
            // $user->save();
            $user_info = $user->userInfo;
            if ($user_info == null) return Base::fail('Verification information not updated');
            // if()
            $user_info->status = $data['status'];
            $user_info->is_vid_verified = $data['status'] == 'verified' ? true : false;
            $user_info->save();
            $user->refresh();
            $data = new UserResource($user);

            //!TODO: Send notification to user if verified.

            return Base::pass("User verification information updated", $data); // DONE
        } catch (Exception $e) {
            return Base::exception($e);
        }
    }

    public function adminUserUpdate($request)
    {
        try {
            $user = User::find($request['user_id']);
            if (!$user) return Base::error("User not found"); // DONE

            $data = $this->updateUser($user, $request);
            return $data->success ? Base::pass($data->message, $data->data) : Base::fail($data->message);
        } catch (Exception $e) {
            return Base::exception($e);
        }
    }

    public function adminAddCustomer($request)
    {
        try {
            // $user = User::find($request['user_id']);
            // if(!$user) return Base::error("User not found.");
            $request['role'] = 'customer';
            $data = $this->createUser($request);
            if ($data->success == true) {
                $user = $data->data;
                if (isset($request['image_url'])) {
                    $img = GlobalTrait::imageUpload($request['image_url'], 'users');
                    $user->image_url = $img;
                }
                $user->is_verified = true;
                $user->phone = isset($request['phone']) ? $request['phone'] : null;
                $user->address = isset($request['address']) ? $request['address'] : null;
                $user->post_code = isset($request['post_code']) ? $request['post_code'] : null;
                $user->city = isset($request['city']) ? $request['city'] : null;
           
                $user->save();
            }
            return $data->success ? Base::pass($data->message, $data->data) : Base::fail($data->message);
        } catch (Exception $e) {
            return Base::exception($e);
        }
    }

    public function vIDVerify($request)
    {
        try {
            $user = Auth::user();

            $user_info = $user->userInfo;

            if ($user_info == null) $user_info = new UserInfo();

            $user_info->user_id = $user->id;
            $user_info->vid_front = GlobalTrait::imageUploadID($request['vid_front'], 'users', $user->id);
            $user_info->vid_back = GlobalTrait::imageUploadID($request['vid_back'], 'users', $user->id);
            $user_info->is_vid_verified = false;
            $user_info->status = 'pending';
            $user_info->type = $request['type'];
            $user_info->save();

            return Base::pass('User verification id uploaded successfully');
        } catch (Exception $e) {
            return Base::exception_fail($e);
        }
    }

    public function vIDVerifyInfo($request)
    {
        try {
            // $auth = Auth::user()->id;

            $user = UserInfo::where('user_id', Auth::user()->id)->first();

            return Base::pass('User verification info', $user);
        } catch (Exception $e) {
            return Base::exception_fail($e);
        }
    }

    public function referralUser($request)
    {
        try {
            // return $request;

            $user = User::where('refer_code', $request['referral_code'])->first(['id', 'name', 'email', 'phone', 'image_url']);

            if($user == null) return Base::fail('Invalid referral code');

            return Base::pass('User general info', $user);
        } catch (Exception $e) {
            return Base::exception_fail($e);
        }
    }

    public function pointDetails($request){
        try{
            $user = Auth::user();
            $data = $user->pointDetails ?? [];

            return Base::pass('User point details', $data);

        }
        catch (Exception $e) {
            return Base::exception_fail($e);
        }
    }
}
